!This program Extract traces from 1D wavefield

!Synopsis < in=1Dwfld.H par=  > traceout.H
! Input: 1Dwfld.H : 1 dimension wavefield file
!                   1=time 2=rz
! Param:    ntout, dtout, otout
!           twin = time window length
!           vel = constant velocity 
! Output:
!         traceout.H  
!         1=time (length of twin), 2=tout (by ntout, dtout)

Program ExtractTrace

  use sep
  implicit none
  
  logical :: verb
  integer    :: ntout, itout, it, nt, nrz, irz, ntwin
  integer    :: itb, ite
  real       :: dtout, otout, dt, ot, orz, drz
  real       :: vel, twin, tdelay, rz, tout
  real, allocatable, dimension(:,:) :: dat
  real, allocatable, dimension(:,:) :: outArr
  character(len=128)              :: outfile_
  


  call sep_init()
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  !Program read-ins
  !velocity model is entered as standard input
  call from_history("n1", nt)
  call from_history("o1", ot)
  call from_history("d1", dt)
  call from_history("n2", nrz)
  call from_history("o2", orz)
  call from_history("d2", drz)

  ! read parameter
  call from_param("ntout",ntout,5); call from_param('otout',otout,0.); call from_param("dtout",dtout,1.);
  call from_param("vel",vel) 
  call from_param("tdelay",tdelay); call from_param('twin',twin,1.);

  call from_param("outfile",outfile_)

  ntwin = (twin/dt)+0.5

  write(0,*) '****** ExtractTrace.f90******'
  write(0,*) 'ntout dtout otout', ntout, dtout, otout
  write(0,*) 'nt dt ot', nt, dt, ot
  write(0,*) 'vel tdelay twin', vel, tdelay, twin
  write(0,*) '*****************************'

  ! Memory allocation
  allocate (dat(nt,nrz), outArr(ntwin,ntout))
  call sep_read(dat)
 
  ! Not calculate
  do itout=1,ntout
     tout= (itout-1)*dtout+otout
     rz = tout*vel
     irz = (rz-orz)/drz+1.5
     itb = (tout-tdelay-ot)/dt+1.5
     ite = itb+ntwin-1
     write(0,*)'itout tout', itout, tout, itb, ite
     if ( ite .lt. itb) call erexit('exit now ite less than itb')
     if ((irz .ge. 1) .and. (irz .le. nrz) .and. (itb .ge. 1) .and. (ite .le. nt) ) then   
        write(0,*) '1st slot' 
      outArr(:,itout) = dat(itb:ite,irz)
     else
        write(0,*) '2nd slot'
        outArr(:,itout) = 0.

     end if
  end do

  ! Output data
    call auxputch("n1",'i',ntwin,outfile_)
    call auxputch("n2",'i',ntout,outfile_)
    call auxputch("d1",'f',dt,outfile_)
    call auxputch("d2",'f',dtout,outfile_)
    call auxputch("o1",'f',ot,outfile_)
    call auxputch("o2",'f',otout,outfile_)
    call auxputch("label1",'s','t(s)',outfile_) 
    call auxputch("label2",'s','tout(s)',outfile_) 
  call sep_write(outArr,outfile_)

 
  call sep_close()

end program
