program SecondDer3D

! Take Second derivative of input file
! It has 3 options
! VDer =  [0] vertical derivative only (d2/dz2)
! lpfil = [0] take laplacian 
! gauss = [0] take gaussian version of laplacian
! only one of 'VDer', 'lpfil', and 'gauss' should be on.
! if not, 'gauss' take precedence over 'lpfil', 
! and 'lpfil' take precendence over 'VDer'

  use sep
  use waveprop3d_mod


implicit none


 logical    :: adj, VDer, lpfil, gauss
 integer    :: nx, nz, ny
 real       :: dx, dz, dy
 real       :: ox, oz, oy
 real, dimension(:), allocatable  ::  temp
 real, dimension(:,:,:), allocatable :: model

 call sep_init()


   write(0,*) ' This is SecondDer3D program'


   ! read in type of filter
   call from_param("adj",adj,.true.);
   call from_param("lpfil",lpfil, .false.);
   call from_param("VDer",VDer, .false.);         
   call from_param("gauss",gauss, .false.);



  ! Read back ground mmocity param
   call from_history("n1", nz);  call from_history("d1", dz);  call from_history("o1",oz);
   call from_history("n2", nx);  call from_history("d2", dx);  call from_history("o2",ox);
   call from_history("n3", ny,1);  call from_history("d3", dy,1.);  call from_history("o3",oy,0.);
  


   nz = nz;
   nx = nx;
   ny = ny;

   write(0,*) 'nz,dz,oz', nz, dz, oz
   write(0,*) 'nx,dx,ox', nx, dx, ox
   write(0,*) 'ny,dy,oy', ny, dy, oy
   write(0,*) 'gauss, lpfil, VDer', gauss, lpfil, VDer
   write(0,*) 'adj', adj


   allocate( model(nz,nx,ny))


write(0,*) 'pass 1'

   ! Read in the model
    call sep_read( model(1:nz,1:nx,1:ny))

    if (gauss) then
	 call Gaussian3D(adj,model,nz,nx,ny)
    else if (lpfil) then
	 call  Laplacian3D(adj,model,nz,nx,ny)
    else if (VDer) then
        call Ver2ndDer3D(adj,model,nz,nx,ny)
    end if

write(0,*) 'pass 3'
   
!----------------------------------------------
! Output

  call sep_write(model,dim1=nz, dim2=nx, dim3=ny, esize=4)

write(0,*) 'pass 4'
  
  call to_history("n1",nz); call to_history("d1",dz);  call to_history("o1",oz);
  call to_history("n2",nx); call to_history("d2",dx);  call to_history("o2",ox);
  call to_history("n3",ny); call to_history("d3",dy);  call to_history("o3",oy);
  
write(0,*) 'pass 5'

end program
