!This program creates synthetic data 1D in T-K domain

!Synopsis < in=velocity aux=source.H  > recdata.H
! dtout=[real] ntout=[int] : time-axis of data  
! zsrc=[real] : Source position
! records nrz at nrt,drt,ort 


Program Synth_data1DTK

  use sep
  use prop_mod1DTK
  use ifport
  implicit none
  
  logical(4) :: success, wfldout
  integer    :: order   ! [2] 2nd order in time and [4] 4th order in time
  real :: otout, dtout,oz,dz, vmax,tdelay,twin
  real :: dst,ost,zsrc, dt_prop, otwin, velconst
  integer :: npad,ntout,nst, nnz,nz,nop, ntwin
  real, allocatable, dimension(:,:) :: dat
  real, allocatable, dimension(:) :: wavelet,vel
  character(len=128)              :: outfile_, wfldfile_
  


  call sep_init()
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  !Program read-ins
  !velocity model is entered as standard input
  call from_history("n1", nz)
  call from_history("o1", oz)
  call from_history("d1", dz)

  
  !the auxiliary file is the source wavelet
  call from_aux("aux", "n1", nst)
  call from_aux("aux", "o1", ost)
  call from_aux("aux", "d1", dst)

  ! read source location 
  call from_param("zsrc",zsrc,dz/2.) 

  ! read data parameter
  call from_param("dtout",dtout,0.004)
  call from_param("ntout",ntout,500)
  call from_param("otout",otout,0.)
  call from_param("tdelay",tdelay); call from_param("twin",twin,2.);
  call from_param("dt_prop",dt_prop,0.001)
  call from_param("npad",npad,30)
  call from_param("order",order,2);

  nnz=nz-2*npad
  ntwin = twin/dt_prop

  ! rather or not to output source wavefield movie
  call from_param("outfile",outfile_); 
  call from_param("wfldout",wfldout,.false.);
  if (wfldout) call from_param("wfldfile",wfldfile_)


  call from_param("vmax",vmax); call from_param("velconst",velconst,vmax)


  dtout=ceiling(1000*dtout)/1000.
  dst=ceiling(1000*dst)/1000.

  write(0,*) '******* Synthout1D **********'
  write(0,*) 'dst, dtout, dt_prop', dst, dtout, dt_prop
  write(0,*) mod(dtout/dt_prop,1.)
  write(0,*) 'twin ntwin', twin, ntwin
  write(0,*) '*******************************'
 ! call erexit('exit now')
  if ( mod(dtout/dt_prop,1.) .gt. 0) call erexit('dtout should be multiple of dt_prop') 
  if ( mod(dst/dt_prop,1.) .gt. 0) call erexit('dst should be multiple of dt_prop') 

 ! success = setenvqq("DATAPATH=/tmp/")

  ! Memory allocation
  allocate (vel(nz),wavelet(nst), dat(ntwin,ntout))

  ! Read velocity model
  call sep_read(vel(1:nz));  call from_param("vmax",vmax);
  ! Read source wavelet
  wavelet=0.
  call sep_read(wavelet,"aux")
 
   write(0,*) 'tdelay', tdelay

  ! Initiate propagation
  call init_propagate(vel,oz,dz,dst, dt_prop,ntout,dtout,otout,tdelay,vmax,npad,wfldout,wfldfile_) 

  ! Initiate source parameters
  call source_init(wavelet,zsrc,tdelay)
  
  ! Initiate receiver parameters
  call receiver_init(ntwin,velconst)
  
  ! do the propagation
  call model_shot(dat,order) 
  
  ! Output data
    call auxputch("n1",'i',ntwin,outfile_)
    call auxputch("n2",'i',ntout,outfile_)
    call auxputch("d1",'f',dt_prop,outfile_)
    call auxputch("d2",'f',dtout,outfile_)
    call auxputch("o1",'f',otout,outfile_)
    call auxputch("o2",'f',otout,outfile_)
    call auxputch("label1",'s','twin(s)',outfile_) 
    call auxputch("label2",'s','tout(s)',outfile_) 
  call sep_write(dat,outfile_)

 
  call sep_close()

end program
