!This program creates synthetic data 1D in T-K domain

!Synopsis < in=velocity aux=source.H  > recdata.H
! dtrec=[real] ntrec=[int] : time-axis of data  
! zsrc=[real] : Source position
! records nrz at nrt,drt,ort 


Program Synth_data2DTK

  use sep
  use prop_mod2DTK
  use ifport
  implicit none
  
  logical(4) :: success, wfldout
  integer    :: order   ! [2] 2nd order in time and [4] 4th order in time
  integer    :: nx, nrx, iz, ix, nnx
  real :: otrec, dtrec,oz,dz, vmax,tdelay
  real :: ox, dx, orx, drx, zsrc, xsrc, zrec
  real :: dst,ost,dt_prop, velconst
  integer :: npad,ntrec,nst, nnz,nz,nop
  real, allocatable, dimension(:,:) :: dat, vel
  real, allocatable, dimension(:) :: wavelet
  character(len=128)              :: outfile_, wfldfile_
  


  call sep_init()
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  !Program read-ins
  !velocity model is entered as standard input
  call from_history("n1", nz)
  call from_history("o1", oz)
  call from_history("d1", dz)
  call from_history("n2", nx)
  call from_history("o2", ox)
  call from_history("d2", dx)
  
  !the auxiliary file is the source wavelet
  call from_aux("aux", "n1", nst)
  call from_aux("aux", "o1", ost)
  call from_aux("aux", "d1", dst)

  ! read source location 
  call from_param("zsrc",zsrc,dz/2.) 
  call from_param("xsrc",xsrc,dx/2.) 

  ! read data parameter
  call from_param("dtrec",dtrec,0.004)
  call from_param("ntrec",ntrec,500)
  call from_param("otrec",otrec,0.)
  call from_param("nrx",nrx,nx);  call from_param("orx",orx,ox); call from_param("drx",drx,dx);
  call from_param("zrec",zrec,zsrc);
  call from_param("tdelay",tdelay);
  call from_param("dt_prop",dt_prop,0.001)
  call from_param("npad",npad,30)
  call from_param("order",order,2);

  nnz=nz
  nnx=nx
  nz = nnz+2*npad
  nx = nnx+2*nop

  ! rather or not to output source wavefield movie
  call from_param("outfile",outfile_); 
  call from_param("wfldout",wfldout,.false.);
  if (wfldout) call from_param("wfldfile",wfldfile_)


  call from_param("vmax",vmax); call from_param("velconst",velconst,vmax)


  dtrec=ceiling(1000*dtrec)/1000.
  dst=ceiling(1000*dst)/1000.

  write(0,*) '******* Synthout2D **********'
  write(0,*) 'dst, dtrec, dt_prop', dst, dtrec, dt_prop
  write(0,*) mod(dtrec/dt_prop,1.)
  write(0,*) '*******************************'
 ! call erexit('exit now')
  if ( mod(dtrec/dt_prop,1.) .gt. 0) call erexit('dtrec should be multiple of dt_prop') 
  if ( mod(dst/dt_prop,1.) .gt. 0) call erexit('dst should be multiple of dt_prop') 

 ! success = setenvqq("DATAPATH=/tmp/")

  ! Memory allocation
  allocate (vel(nz,nx),wavelet(nst), dat(ntrec,nrx))

  ! Read velocity model
  call sep_read(vel(1+npad:nz-npad,1+nop:nx-nop));  call from_param("vmax",vmax);
  ! Pad the velocity
  do iz=1,npad
     vel(iz,:) = vel(npad+1,:)
     vel(nz-iz+1,:) = vel(nz-npad,:)
  end do
  do ix=1,nop
     vel(:,ix) = vel(:,nop+1)
  end do
  do ix=1,nop
     vel(:,nx-ix+1) = vel(:,nx-nop)
  end do


  ! Read source wavelet
  wavelet=0.
  call sep_read(wavelet,"aux")
 
   write(0,*) 'tdelay', tdelay

  ! Initiate propagation
  call init_propagate(vel,oz,dz,ox,dx,dst, dt_prop,ntrec,dtrec,otrec,tdelay,vmax,npad,wfldout,wfldfile_) 

  ! Initiate source parameters
  call source_init(wavelet,zsrc,xsrc,tdelay)
  
  ! Initiate receiver parameters
  call  receiver_init(zrec,nrx,orx,drx)
  
  ! do the propagation
  call model_shot(dat,order,outfile_) 
  
  if (order .eq. 2) then
  ! Output data
    call auxputch("n1",'i',ntrec,outfile_)
    call auxputch("n2",'i',nrx,outfile_)
    call auxputch("d1",'f',dtrec,outfile_)
    call auxputch("d2",'f',drx,outfile_)
    call auxputch("o1",'f',otrec,outfile_)
    call auxputch("o2",'f',orx,outfile_)
    call auxputch("label1",'s','t(s)',outfile_) 
    call auxputch("label2",'s','x(m)',outfile_) 
  call sep_write(dat,outfile_)
  end if
 
  call sep_close()

end program
