! This module create 1D filters to match two waveforms
! input : original waveform
! match: target waveform
! filter: desired filter 

module create_filter_mod
  use base_fftwv3_mod
  use hamming_window_mod
  use sep
  implicit none

  integer, private  :: nw1, nw2, nw3, nw4
  integer, private  :: nt, nw 
  real, private     :: ot, dt, ow, dw, eps
  logical, private  :: wnttap
  complex, dimension(:), allocatable, private :: ffilter
  complex, dimension(:), allocatable, private :: finput
  complex, dimension(:), allocatable, private :: fmatch

contains

  integer function create_filter_init(nt_in, ot_in, dt_in, f1, f2, f3, f4, radian, eps_in, wnttap_in) result(stat)

    integer  :: nt_in
    real     :: ot_in, dt_in, eps_in
    real     :: f1, f2, f3, f4, w1, w2, w3, w4
    logical  :: radian, wnttap_in
    real, parameter :: PI = 3.14159265359

    nt = nt_in
    ot = ot_in
    dt = dt_in

    eps = eps_in
    wnttap = wnttap_in

    if (radian) then
       w1=2*PI*f1
       w2=2*PI*f2
       w3=2*PI*f3
       w4=2*PI*f4
    else
       w1=f1
       w2=f2
       w3=f3
       w4=f4 
    end if

    dw = 1./(nt*dt)
    if (radian) dw = dw * 2 * PI

    nw1=w1/dw+1.5
    nw2=w2/dw+1.5
    nw3=w3/dw+1.5
    nw4=w4/dw+1.5 
    ow = 0. 
    nw = nt 

    allocate(finput(nw), fmatch(nw), ffilter(nw))
    finput  = 0.
    fmatch  = 0.
    ffilter = 0.

    stat = 0
  end function

  subroutine create_filter(input, match, filter, apply)

    integer*8:: plan_1dt1, plan_1dt2, plan_1dt3
    real   , dimension(:) :: input
    real   , dimension(:) :: match
    real   , dimension(:) :: filter 
    integer               :: it
    logical               :: apply
  
    finput = 0; fmatch = 0; ffilter = 0;

    call sfftw_dft_1dt_init(plan_1dt1, finput , nt, 1)
    call sfftw_dft_1dt_init(plan_1dt2, fmatch , nt, 1)
    call sfftw_dft_1dt_init(plan_1dt3, ffilter, nt,-1)

    ! fft for both signals
    finput = input; fmatch = match; 
    call sfftw_dft_1dt_oper(plan_1dt1, finput, nt)
    call sfftw_dft_1dt_oper(plan_1dt2, fmatch, nt)

    if (apply) then
      ! apply the filter
      ffilter = fmatch * finput
    else
      ! filter spectrum is the ratio of match over input
      ffilter = fmatch / (finput + eps)
    end if

    ! band limit the range of the frequencies
    if (wnttap) then
      call hammingw(ffilter, nw1, nw2, nw3, nw4)
    end if  

    ! inverse ft to get the filter in time
    call sfftw_dft_1dt_oper(plan_1dt3, ffilter, nt)

    ! ifft shift
    do it=1,nt/2
      filter(it)      = real(ffilter(nt/2+it))
      filter(nt/2+it) = real(ffilter(it))
    end do

  end subroutine

  integer function create_filter_clos() result(stat)

    deallocate(finput, fmatch, ffilter)
    stat = 0
  end function

  integer function pad2(nin) result(nout)
    integer nin
    integer i
    nout = 2;
    do while (nout .le. nin)
      nout = nout * 2
    end do
  end function
end module 
