! This program filters a record using discrete filters 
! and gets the final record by interpolation  
! input: data record 
! match: target waveform
! output: data record without time dispersion 

program main
  use base_fftwv3_mod
  use hamming_window_mod
  use create_filter_mod
  use sep
  implicit none

  integer  :: nw1, nw2, nw3, nw4
  integer  :: ix, it
  integer  :: nt, nw, nx, ntau 
  integer*8:: plan_1dt1, plan_1dt2, plan_1dt3
  real     :: ot, ow, ox, otau, ot_pad
  real     :: dt, dw, dx, dtau, eps
  real     :: f1, f2, f3, f4, w1, w2, w3, w4
  integer  :: nfilter, ifilter
  real     :: dfilter, ofilter
  real   , dimension(:,:)    , allocatable :: filter
  real   , dimension(:,:,:)  , allocatable :: input
  real   , dimension(:)      , allocatable :: tmp1, tmp2, tmp3
  logical  :: adj
  logical  :: radian
  logical  :: wnttap
  real, parameter :: PI = 3.14159265359
  integer  :: stat, npad, nbegd, nbegf
  real     :: iw1, iw2, xx, xx1, xx2     ! interpolation weights


  call initpar()

  call from_param("f1", f1,  5.)
  call from_param("f2", f2, 10.)
  call from_param("f3", f3, 35.)
  call from_param("f4", f4, 40.)

  call from_param("adj", adj, .true.)
  call from_param("eps", eps, 0.0001)
  call from_param("radian", radian, .true.)
  call from_param("wnttap", wnttap, .true.)

  call from_history("n1",  nt); call from_history("d1",  dt); call from_history("o1",  ot)
  call from_history("n2",  nx); call from_history("d2",  dx); call from_history("o2",  ox)

  call from_aux("filter", "d1", dtau)
  call from_aux("filter", "o1", otau)
  call from_aux("filter", "n1", ntau)
  call from_aux("filter", "d2", dfilter)
  call from_aux("filter", "o2", ofilter)
  call from_aux("filter", "n2", nfilter)

  if (dtau .ne. dt) then
    write(0,*) "filter sampling is different from data sampling"
    stop
  end if


  allocate(input(nt,nx,nfilter+1), filter(ntau,nfilter))
  input = 0; filter = 0;
  call sreed("in", input(:,:,1), 4*nt*nx)
  call sreed("filter", filter, 4*ntau*nfilter)

  npad = max(pad2(nt*5/4),pad2(ntau))
  if (nt   .ne. npad) write(0,*) "pad data to", npad
  if (ntau .ne. npad) write(0,*) "pad filter to", npad
  nbegd = (npad - nt  ) / 2
  nbegf = (npad - ntau) / 2
  allocate(tmp1(npad), tmp2(npad), tmp3(npad))
  tmp1 = 0; tmp2 = 0; tmp3 = 0;

  ot_pad = ot - nbegd*dt
  stat = create_filter_init(npad, ot_pad, dt, f1, f2, f3, f4, radian, eps, wnttap) 

  ! apply the filters to the whole dataset
  do ifilter = 1, nfilter
    tmp2(1+nbegf:ntau+nbegf) = filter(1:ntau,ifilter)
    do ix = 1, nx
      tmp1(1+nbegd:nt+nbegd  ) = input (1:nt, ix,  1)
      call create_filter(tmp1, tmp2, tmp3, .true.)
      input(:,ix,ifilter+1) = tmp3(1+nbegd:nt+nbegd)
    end do
  end do

  ! interpolate between filters to obtain the final results"
  do ix = 1, nx
      xx = (ix-1)*dx + ox 
      ifilter = (xx-ofilter)/dfilter+1
      xx1 = (ifilter -1) * dfilter + ofilter
      xx2 = (ifilter   ) * dfilter + ofilter
      iw1 = (xx-xx1)/(xx2-xx1)
      if (ifilter .le. nfilter) then
        input(:,ix,1) = (1-iw1)*input(:,ix,ifilter) + (iw1)*input(:,ix,ifilter+1)
      else
        input(:,ix,1) = (1-iw1)*input(:,ix,nfilter+1) 
      end if
  end do

  call srite("out", input(:,:,1), 4*nx*nt)
  call to_history("n1",nt)
  call to_history("d1",dt)
  call to_history("o1",ot)
  call to_history("n2",nx)
  call to_history("d2",dx)
  call to_history("o2",ox)

  stat = create_filter_clos()
  
  deallocate(input, filter )

end program
