! This program filters a record using non-stationary filters 
! whose coefficients are obtained by interpolating the existing filters 
! input : data record 
! filter: filters estimated at discrete times 
! output: data record without time dispersion 

program main
  use base_fftwv3_mod
  use hamming_window_mod
  use create_filter_mod
  use sep
  implicit none

  integer  :: nw1, nw2, nw3, nw4
  integer  :: ix, it
  integer*8:: plan_1dt1, plan_1dt2, plan_1dt3
  integer  :: nt, nw, nx, ntau, npad
  real     :: ot, ow, ox, otau, otpad
  real     :: dt, dw, dx, dtau, eps
  real     :: f1, f2, f3, f4, w1, w2, w3, w4
  integer  :: nfilter, ifilter
  real     :: dfilter, ofilter
  real   , dimension(:,:)    , allocatable :: filter
  real   , dimension(:,:)    , allocatable :: input, output
  real   , dimension(:)      , allocatable :: tmp1,tmp2,tmp3
  logical  :: adj
  logical  :: radian
  logical  :: wnttap
  real, parameter :: PI = 3.14159265359
  integer  :: stat
  real     :: iw1, iw2, tt, tt1, tt2     ! interpolation weights
  integer  :: itb, ite, itl, nbegf


  call initpar()

  call from_param("f1", f1,  5.)
  call from_param("f2", f2, 10.)
  call from_param("f3", f3, 35.)
  call from_param("f4", f4, 40.)

  call from_param("adj", adj, .true.)
  call from_param("eps", eps, 0.0001)
  call from_param("radian", radian, .true.)
  call from_param("wnttap", wnttap, .true.)

  call from_history("n1",  nt); call from_history("d1",  dt); call from_history("o1",  ot)
  call from_history("n2",  nx); call from_history("d2",  dx); call from_history("o2",  ox)

  call from_aux("filter", "d1", dtau)
  call from_aux("filter", "o1", otau)
  call from_aux("filter", "n1", ntau)
  call from_aux("filter", "d2", dfilter)
  call from_aux("filter", "o2", ofilter)
  call from_aux("filter", "n2", nfilter)

  if (dtau .ne. dt) then
    write(0,*) "filter sampling is different from data sampling"
    stop
  end if


  allocate(input(nt,nx), filter(ntau,nfilter), output(nt,nx))
  input = 0; filter = 0; output = 0
  call sreed("in", input(:,:), 4*nt*nx)
  call sreed("filter", filter, 4*ntau*nfilter)

  ! pad the filter for fft
  npad = pad2(ntau*2)
  if (ntau .ne. npad) write(0,*) "pad filter to", npad
  nbegf = (npad - ntau) / 2
  allocate(tmp1(npad), tmp2(npad), tmp3(npad))
  tmp1 = 0; tmp2 = 0; tmp3 = 0;
  
  ! initialize the filter
  stat = create_filter_init(npad, ot, dt, f1, f2, f3, f4, radian, eps, wnttap) 

  ! interpolate between filters to obtain the final results
  do ix = 1, nx
    do it = 1, nt, ntau/8
      ! prepare the data patch
      itb = max(1,it-ntau/2+1)
      ite = min(nt,it+ntau/2)
      itl = ite - itb + 1
      tmp1 (nbegf+ntau-itl+1:nbegf+ntau) = input(itb:ite,ix)
      ! prepare the filter  
      tt = (it-1)*dt + ot 
      ifilter = (tt-ofilter)/dfilter+1
      tt1 = (ifilter -1) * dfilter + ofilter
      tt2 = (ifilter   ) * dfilter + ofilter
      iw1 = (tt-tt1)/(tt2-tt1)
      if (ifilter .lt. nfilter) then
        tmp2(1+nbegf:ntau+nbegf) = (1-iw1)*filter(:,ifilter) + (iw1)*filter(:,ifilter+1)
      else
        tmp2(1+nbegf:ntau+nbegf) = (1-iw1)*filter(:,nfilter) 
      end if
      ! apply the filter
      call create_filter(tmp1, tmp2, tmp3, .true.)
      output(itb:ite,ix) = tmp3(nbegf+ntau-itl+1:nbegf+ntau) + output(itb:ite,ix)  
    end do
  end do

  output = output / 8

  call srite("out", output(:,:), 4*nx*nt)
  call to_history("n1",nt)
  call to_history("d1",dt)
  call to_history("o1",ot)
  call to_history("n2",nx)
  call to_history("d2",dx)
  call to_history("o2",ox)

  stat = create_filter_clos()
  
  deallocate(input, output, tmp1, tmp2, tmp3, filter )

end program
