! This program filters a record using non-stationary filters 
! whose coefficients are obtained by interpolating the existing filters 
! input : data record 
! filter: filters estimated at discrete times 
! output: data record without time dispersion 

program main
  use base_fftwv3_mod
  use hamming_window_mod
  use create_filter_mod
  use sep
  implicit none

  integer  :: nw1, nw2, nw3, nw4
  integer  :: ix, it
  integer  :: nt, nw, nx 
  integer*8:: plan_1dt1, plan_1dt2, plan_1dt3
  real     :: ot, ow, ox
  real     :: dt, dw, dx, eps
  real     :: f1, f2, f3, f4, w1, w2, w3, w4
  integer  :: nfilter, ifilter
  real     :: dfilter, ofilter
  real   , dimension(:,:)    , allocatable :: filter
  real   , dimension(:,:)    , allocatable :: input, output
  real   , dimension(:)      , allocatable :: tmpft
  logical  :: adj
  logical  :: radian
  logical  :: wnttap
  real, parameter :: PI = 3.14159265359
  integer  :: stat
  real     :: iw1, iw2, xx, xx1, xx2     ! interpolation weights


  call initpar()

  call from_param("f1", f1,  5.)
  call from_param("f2", f2, 10.)
  call from_param("f3", f3, 35.)
  call from_param("f4", f4, 40.)

  call from_param("adj", adj, .true.)
  call from_param("eps", eps, 0.0001)
  call from_param("radian", radian, .true.)
  call from_param("wnttap", wnttap, .true.)

  call from_history("n1",  nt); call from_history("d1",  dt); call from_history("o1",  ot)
  call from_history("n2",  nx); call from_history("d2",  dx); call from_history("o2",  ox)

  call from_aux("filter", "d2", dfilter)
  call from_aux("filter", "o2", ofilter)
  call from_aux("filter", "n2", nfilter)

  stat = create_filter_init(nt, ot, dt, f1, f2, f3, f4, radian, eps, wnttap) 

  allocate(input(nt,nx), output(nt,nx), filter(nt,nfilter), tmpft(nt))
  input = 0;  filter = 0;
  call sreed("in"    , input , 4*nx*nt     )
  call sreed("filter", filter, 4*nt*nfilter)
  
  ! interpolate between filters to obtain the final results
  do ix = 1, nx
    xx = (ix-1)*dx + ox 
    ifilter = (xx-ofilter)/dfilter+1
    xx1 = (ifilter -1) * dfilter + ofilter
    xx2 = (ifilter   ) * dfilter + ofilter
    iw1 = (xx-xx1)/(xx2-xx1)
    if (ifilter .lt. nfilter) then
      tmpft = (1-iw1)*filter(:,ifilter) + iw1*filter(:,ifilter+1)
    else
      tmpft = (1-iw1)*filter(:,nfilter) 
    end if
    call create_filter(input(:,ix), tmpft, output(:,ix), .true.)
  end do

  call srite("out", output(:,:), 4*nx*nt)
  call to_history("n1",nt)
  call to_history("d1",dt)
  call to_history("o1",ot)
  call to_history("n2",nx)
  call to_history("d2",dx)
  call to_history("o2",ox)

  stat = create_filter_clos()
  
  deallocate(input, output, tmpft, filter )

end program
