module fourier_modOMP

  implicit none
  include "fftw3.f"

  integer, private                             :: n1, n2
  integer*8, private                           :: plan1f, plan1b, plan2f, plan2b
  contains

  subroutine ft_init(n1_in, n2_in,in1,out1,in2,out2)
    integer               :: n1_in, n2_in
    complex, dimension(:)  :: in1,out1,in2,out2    

    n1 = n1_in
    n2 = n2_in
    call sfftw_plan_dft_1d(plan1f, n1, in1, out1, FFTW_FORWARD, FFTW_ESTIMATE)
    call sfftw_plan_dft_1d(plan1b, n1, out1, in1, FFTW_BACKWARD, FFTW_ESTIMATE)
    call sfftw_plan_dft_1d(plan2f, n2, in2, out2, FFTW_FORWARD, FFTW_ESTIMATE)
    call sfftw_plan_dft_1d(plan2b, n2, out2, in2, FFTW_BACKWARD, FFTW_ESTIMATE)
  end subroutine

  subroutine ft1axis(adj, sign1, cdata,in1,out1)
    logical, intent(in)        :: adj
    integer                    :: i2
    complex, dimension(:,:) :: cdata
    real                       :: sign1
    complex, dimension(:)  :: in1,out1    

    if(adj) then
      do i2=1,n2
        out1(1:(n1+1)/2) = cdata(n1-(n1+1)/2+1:n1,i2)
        out1((n1+1)/2+1:n1) = cdata(1:n1-(n1+1)/2,i2)
        call sfftw_execute_dft(plan1b, out1, in1)
        cdata(:,i2) = in1
      enddo
    else
      do i2=1,n2
        in1 = cdata(:,i2)
        call sfftw_execute_dft(plan1f, in1, out1)
        cdata(:,i2) = out1
        cdata(1:n1-(n1+1)/2,i2) = out1((n1+1)/2+1:n1)
        cdata(n1-(n1+1)/2+1:n1,i2) = out1(1:(n1+1)/2)
      enddo
    endif
    cdata = sign1*cdata/sqrt(1.*n1)
  end subroutine

  subroutine ft2axis(adj, sign2, cdata,in2,out2)
    logical, intent(in)        :: adj
    integer                    :: i1
    complex, dimension(:,:) :: cdata
    real                       :: sign2
    complex, dimension(:)  :: in2,out2    

    if(adj) then
      do i1=1,n1
        out2(1:(n2+1)/2) = cdata(i1,n2-(n2+1)/2+1:n2)
        out2((n2+1)/2+1:n2) = cdata(i1,1:n2-(n2+1)/2)
        call sfftw_execute_dft(plan2b, out2, in2)
        cdata(i1,:) = in2
      enddo
    else
      do i1=1,n1
        in2 = cdata(i1,:)
        call sfftw_execute_dft(plan2f, in2, out2)
        cdata(i1,:) = sign2*out2/sqrt(1.*n2)
        cdata(i1,1:n2-(n2+1)/2) = out2((n2+1)/2+1:n2)
        cdata(i1,n2-(n2+1)/2+1:n2) = out2(1:(n2+1)/2)
      enddo
    endif
    cdata = sign2*cdata/sqrt(1.*n2)
  end subroutine

end module
