module timeDer_mod

  use sep

  implicit none
  logical      :: verbb


contains

!==========================================================

  subroutine  timeDer_init(verb_in)

    logical  :: verb_in
  
    verbb = verb_in
  end subroutine

!==========================================================

  subroutine firstDerLeft(add,adj,m,mDer,nt,dt)
  ! Take left-handed first derivative
  !  fDer_(i) = (f(i+1)-f(i))/dt 

  logical                   :: add, adj
  integer                   :: nt, it
  real                      :: dt
  real, dimension(:), allocatable :: mtmp
  real, dimension(:)      :: m, mDer

  if (verbb) then
  write(0,*) 'first Derivative left'
  write(0,*) ' nt, dt, adj,add', nt, dt, adj,add
  end if

  allocate(mtmp(nt))
  if (nt .ne. size(m,1)) call erexit('length of nt not equal to size input wavlet')
  if (nt .ne. size(mDer,1)) call erexit('length of nt not equal to size input deri wavlet')
  if (dt .le. 0.) call erexit('dt is less than zero')  

  !! initialize
  if (.not. add) then
     if (.not. adj) then
        do it=1,nt
           mDer(it) = 0.
        end do
    else
        do it=1,nt
           mDer(it) = 0.
        end do
    end if
  end if

  
  if (.not. adj) then
      do it=1,nt-1
         mDer(it) = mDer(it)+ (m(it+1)-m(it))/dt
      end do
  else
      do it=1,nt-1
         m(it+1) = m(it+1)+mDer(it)/dt
         m(it) = m(it)-mDer(it)/dt
      end do
  end if

  end subroutine 
!==========================================================
  subroutine firstDerRight(add,adj,m,mDer,nt,dt)
  ! Take right-handed first derivative
  !  fDer_(i) = (f(i)-f(i-1))/dt 

  logical                   :: add, adj
  integer                   :: nt, it
  real                      :: dt
  real, dimension(:), allocatable :: mtmp
  real, dimension(:)      :: m, mDer

  if (verbb) then
  write(0,*) 'first Derivative right'
  write(0,*) ' nt, dt, adj, add', nt, dt, adj, add
  end if

  allocate(mtmp(nt))
  if (nt .ne. size(m,1)) call erexit('length of nt not equal to size input wavlet')
  if (nt .ne. size(mDer,1)) call erexit('length of nt not equal to size input deri wavlet')
  if (dt .le. 0.) call erexit('dt is less than zero')  

  !! initialize
  if (.not. add) then
     if (.not. adj) then
        do it=1,nt
           mDer(it) = 0.
        end do
    else
        do it=1,nt
           mDer(it) = 0.
        end do
    end if
  end if

  
 
  if (.not. adj) then
      do it=1+1,nt
         mDer(it) = mDer(it)+ (m(it)-m(it-1))/dt
      end do
  else
      do it=1,nt-1
         m(it-1) = m(it-1)-mDer(it)/dt
         m(it) = m(it)+mDer(it)/dt
      end do
  end if

  end subroutine 
!==========================================================
  subroutine SecondDer(add,adj,m,mDer,nt,dt)
  ! Take Second Derivatives
  !  fDer_(i) = (f(i+1)-2*f(i)-f(i-1))/dt**2 

  logical                   :: add, adj
  integer                   :: nt, it
  real                      :: dt, idt
  real, dimension(:), allocatable :: mtmp
  real, dimension(:)      :: m, mDer

  if (verbb) then
  write(0,*) 'Second Derivatives'
  write(0,*) ' nt, dt, adj, add', nt, dt, adj, add
  end if

  allocate(mtmp(nt))
  if (nt .ne. size(m,1)) call erexit('length of nt not equal to size input wavlet')
  if (nt .ne. size(mDer,1)) call erexit('length of nt not equal to size input deri wavlet')
  if (dt .le. 0.) call erexit('dt is less than zero')  

  !! initialize
  if (.not. add) then
     if (.not. adj) then
        do it=1,nt
           mDer(it) = 0.
        end do
    else
        do it=1,nt
           mDer(it) = 0.
        end do
    end if
  end if

  !idt = 1./dt/dt
  idt = 1.

  if (.not. adj) then

      do it=1+1,nt-1
         mDer(it) = mDer(it)+ (m(it+1)-2*m(it)+m(it-1))*idt
      end do

  else
      do it=1+1,nt-1
         m(it-1) = m(it-1)+   mDer(it)*idt
         m(it)   = m(it)  + 2*mDer(it)*idt
         m(it+1) = m(it+1) +  mDer(it)*idt
      end do
  end if
 

  end subroutine 
!==================================================================================

   subroutine getfilename(base_,swout_,ifile)

        integer      :: ifile,p1,d2
        character(len=128)   :: base_,swout_
        character*3          :: aa

        p1 = scan(base_,".H")

        if (ifile .eq. 0) then
          swout_(:p1+1) = base_(:p1+1)
        else
  	  swout_(:p1-1) = base_(:p1-1)
          swout_(p1:) = " "
          write(aa,1000) ifile
          1000 format(i3)

          d2=1
          if (ifile .gt. 9) d2=2
          if (ifile .gt. 99) d2=3

          swout_(p1:p1) ="V"
          swout_(p1+1:p1+d2)=aa(3-d2+1:3)
          swout_(p1+d2+1:)=".H"

        end if

   end subroutine

!====================================================================================
end module
