program timeDerivative
! Calculate timeseries with nth-derivatives
! Input: 
!      tmax = maximum time of record
!      dtout = time spacing to output the wavelet
!      wavefile = input wavelet filename
!      outfile = output file name
! Output:
!      i=1,..,N files that corresponds to applying ith derivative on the input files
!      N = tmax/dt

  use sep
  use timeDer_mod


  implicit none
  
    
  logical                              :: verb, start
  integer                              :: nt, it, ik, ii
  integer                              :: ifile, isave, ntout, ncount
  real                                 :: dtout, dt, tmax, ot, oot
  real                                 :: sc, minv, maxv
  real, dimension(:), allocatable      :: fs, fsDer
  character(len=128)                   :: out_, swout_

  call sep_init()

!!======================================================
  !! Parameter reading

   call from_param("verb",verb,.false.);   
   call from_param("dtout",dtout,0.1);
   call from_param("tmax",tmax,5.);
   call from_param("outfile",out_);
   
   ! Read back ground velocity param
   call from_history("n1",nt);  call from_history( "d1",dt);  call from_history( "o1",ot);  
   call from_history( "o1",ot);
  
   call timeDer_init(verb);

   ntout= tmax/dtout+1
   ncount = dtout/dt
   if (ncount .lt. 1) ncount=ncount

   if (verb) then
   write(0,*) '**************************'
   write(0,*) 'wavelet: nt, dt, ot', nt, dt, ot
   write(0,*) 'tout: tmax, dtout', tmax, dtout
   write(0,*) 'output files: ntout, ncount', ntout, ncount
   write(0,*) '**************************'
   end if

 !   call erexit('exit now')
   !! Error checking

   !! Allocate
   allocate(fs(nt),fsDer(nt))  

   !! read wavelet
   call sep_read(fs)
   do it=1,nt
      fsDer(it) = 0.
   end do

 !******************************************************************


   isave = 1
   ifile = 1
   start = .true.
   do it=1,ntout
      oot = (it-1)*dtout
       
     ncount=1
      do ik=1,ncount


        ! Write wavelet to disk and then reset isave to 1
        if  ((start) .or. (isave .ge. (ncount+1))) then
          if (verb) write(0,*) 'write to disk', oot, ifile
          call getfilename(out_,swout_,ifile)
          call auxputch("n1",'i',nt, trim(swout_)); 
          call auxputch("d1",'f', dt, trim(swout_));  
          call auxputch("o1",'f', ot, trim(swout_));   
          call auxputch("n2",'i',1, trim(swout_)); 
          call auxputch("d2",'f', 1., trim(swout_));  
          call auxputch("o2",'f', oot, trim(swout_));  
          call auxputch("label1", 's', 't', trim(swout_));
          call auxputch("label2", 's', 'tout', trim(swout_));
	  call sep_write(fs, trim(swout_),esize=4)
          ! reset the counter
          isave = 1
          ifile = ifile+1
          start=.false.
        end if


         call secondDer(.false.,.false.,fs,fsDer,nt,dt)

         maxv = maxval(fsDer)
         minv = minval(fsDer)
         if (abs(minv) .gt. abs(maxv)) then
             sc = 1./minv
         else
             sc = 1./maxv
         end if
         
         do ii=1,nt
            fs(ii) = fsDer(ii)*sc
         end do


         isave=isave+1


     end do
   end do

!-----------------------------------------------------

   deallocate( fs, fsDer)

   call sep_close()

   write(0,*) 'program should exit'

end program
